#
#    Copyright Analog Flavor 2013-2024
#
#    All rights reserved.
#
#    This software contains proprietary and confidential information which is the
#    property of Analog Flavor and is subject to license terms.
#
#    This file may be used in accordance with the license agreement 
#    provided with the software.
#
#    This copyright notice may not be removed or altered.
#
## @file bspwave_socket.py
#  \brief This file contains an example for the use of the BeSpice Wave interactive mode from a Python application.
#       The current example creates and uses a socket to communicate with BeSpice Wave.
#
#  The executable bspwave is launched in interactive mode in a subprocess.
#  Writing and reading to the sockets allow to use the interface.
#
#    More information on how to launch external commands in Python subprocesses and how to communicate over sockets can be found in
#    the <a href="http://www.python.org/doc/">Python documentation</a>.
#
#  To run the example go to your Analog Flavor install tree.
#
#   \verbatim
#       > cd analog_flavor/examples/bspwave/example
#       > python bspwave_socket.py
#       --> open_file fourbitadder.spi3
#       --> add_plot "plot 2" analog 0
#       --> add_plot "plot 3" analog 0
#       --> add_curve_to_plot fourbitadder.spi3 v(*) "plot 2"
#       --> add_curve_to_plot fourbitadder.spi3 i(*) "plot 3"
#       --> zoom "plot 2" 0% 150% -10% 110% -10% 110%
#       --> set_cursor "plot 2" 10%
#       --> set_cursor "plot 2" 70%
#       --> hide
#       --> show
#       --> x
#   \endverbatim
# @cond

import subprocess
import sys
import socket

# create a socket for the connection to BeSpice Wave
port_index = 35012
try:
    server_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    server_socket.bind(('', port_index))
    server_socket.listen(1)

except socket.error as msg:
    msg = "Failed to create socket on port {0}.".format(port_index)
    print(msg)
    sys.exit(1)

# launch bspwave in interactive socket mode in a parallel subprocess
port_string = "{0}".format(port_index)
bspwave_process=subprocess.Popen(['../../../bin/bspwave', '--socket', '127.0.0.1', port_string])

# wait for BeSpice Wave to connect
conn, addr = server_socket.accept()
conn.setblocking(1)

# on success "0" is returned
output = conn.recv(1024)
if not output:
    print("data reception failed.")
else :
    msg = "received {0} bytes : \"{1}\"".format(len(output), output)
    print(msg)

print("Type \"x return\" to exit.")
print("Some of the available commands : ")
print("    open_file <file_name>")
print("    add_plot <plot_name> <plot_type> <flag_new_page>")
print("    add_curve_to_plot <section_name> <curve_name> <plot_name>")
print("    zoom <plot_name> x0 x1 y00 y01 y10 y11")
print("    set_cursor <plot_name> x0")
print("    show")
print("    hide")
print("    close")

# execution loop
try :
    while True:
        # wait for text input
        if sys.version_info < (3, 0):
            s = raw_input('--> ')
        else:
            s = input('--> ')

        # convert text input to command
        cmd = "{0}\n".format(s)
        if s == "x" :
            # exit
            cmd = "close\n"

        # write interactive command to socket
        if sys.version_info < (3, 0):
            conn.sendall(cmd)
        else:
            conn.sendall(bytes(cmd, 'UTF-8'))

        # receive the value returned over socket
        output = conn.recv(1024)
        if not output:
            print("data reception failed.")
            break
        msg = "received {0} bytes : \"{1}\"".format(len(output), output)
        print(msg)

except socket.error as exc:
    print("Socket error.")

try:
    conn.shutdown(socket.SHUT_RDWR)
    conn.close()

    server_socket.shutdown(socket.SHUT_RDWR)
    server_socket.close()

except socket.error as msg:
    print("Socket error.")

print("Waiting for BeSpice Wave to terminate.")
bspwave_process.wait()
print("Terminated.")

# @endcond
