/*
    Copyright Analog Flavor 2013-2024

    All rights reserved.

    This software contains proprietary and confidential information which is the
    property of Analog Flavor and is subject to license terms.

    This file may be used in accordance with the license agreement 
    provided with the software.

    This copyright notice may not be removed or altered.
*/

/*! \file af_wp_plugin.h
  \brief This file contains the function headers and types required for writing extensions to BeSpice Wave Parser.

  The function must be implement to allow BeSpice Wave Parser to access data handled by the shared libarary.
  This allows BeSpice Wave Parser to work with user defined waveform formats or data bases.

*/

#ifndef __af_wp_plugin__
#define __af_wp_plugin__

#include "af_wp_types.h"

/** \defgroup bspwave_data_plugins Writing Data Plug-ins

    These functions and types are required for writing extensions to BeSpice Wave Parser.

    @{ */

#define AF_WAVEFORM_PARSER_PLUGIN_VERSION 1

#ifdef __cplusplus
extern "C" {
#endif

    typedef void af_wp_plugin_release_type(void *plugin_handle);
    typedef const char *af_wp_plugin_get_error_message_type(void *plugin_handle);
    typedef void *af_wp_plugin_parse_file_type(void *plugin_handle, const char *file_name);
    typedef af_return_code af_wp_plugin_reload_file_type(void *plugin_handle, void *file_handle);
    typedef void af_wp_plugin_get_plugin_property_type(void *plugin_handle, int property_index, af_wp_prop_type *property);

    typedef af_return_code af_wp_plugin_release_file_type(void *plugin_handle, void *file_handle);
    typedef af_return_code af_wp_plugin_release_all_files_type(void *plugin_handle);
    typedef af_return_code af_wp_plugin_get_file_property_type(void *plugin_handle, void *file_handle, int property_index, af_wp_prop_type *property);

    typedef void *af_wp_plugin_get_section_handle_type(void *plugin_handle, void *file_handle, int section_index);
    typedef af_return_code af_wp_plugin_get_section_property_type(void *plugin_handle, void *file_handle, void *section_handle, int property_index, af_wp_prop_type *property);
    typedef af_return_code af_wp_plugin_set_section_property_type(void *plugin_handle, void *file_handle, void *section_handle, const af_wp_prop_type *property);

    typedef af_return_code af_wp_plugin_get_variable_property_type(void *plugin_handle, void *file_handle, void *section_handle, int variable_index, int property_index, af_wp_prop_type *property);
    typedef af_return_code af_wp_plugin_copy_variable_data_type(void *plugin_handle, void *file_handle, void *section_handle, int variable_index, af_wp_data_type data_type, void *data, int data_byte_size);
    typedef af_return_code af_wp_plugin_retrieve_variable_data_type(void *plugin_handle, void *file_handle, void *section_handle, int variable_index, af_wp_data_type *data_type, void **x_data, int *x_data_byte_size, void **y_data, int *y_data_byte_size);

    typedef void *af_wp_plugin_variable_value_iterator_define_type(void *plugin_handle, void *file_handle, void *section_handle, int variable_index);
    typedef af_return_code af_wp_plugin_variable_value_iterator_release_type(void *plugin_handle, void *file_handle, void *section_handle, void *iterator_handle);
    typedef af_return_code af_wp_plugin_variable_value_iterator_read_next_type(void *plugin_handle, void *file_handle, void *section_handle, void *iterator_handle, af_wp_value_type *x_value, af_wp_value_type *y_value);

    typedef af_return_code af_wp_plugin_get_hierarchical_instance_property_type(void *plugin_handle, void *file_handle, void *section_handle, int instance_index, int property_index, af_wp_prop_type *property);
    typedef int af_wp_plugin_get_hierarchical_instance_sub_instance_index_type(void *plugin_handle, void *file_handle, void *section_handle, int instance_index, int i);
    typedef int af_wp_plugin_get_hierarchical_instance_variable_index_type(void *plugin_handle, void *file_handle, void *section_handle, int instance_index, int i);

    typedef af_return_code af_wp_plugin_get_exporter_property_type(void *plugin_handle, const char *exporter_name, int property_index, af_wp_prop_type *property);
    typedef void *af_wp_plugin_exporter_create_type(void *plugin_handle, const char *exporter_name, const char *file_name);
    typedef void af_wp_plugin_exporter_release_type(void *plugin_handle, void *exporter_handle);
    typedef void af_wp_plugin_exporter_set_property_type(void *plugin_handle, void *exporter_handle, const af_wp_prop_type *property);
    typedef void af_wp_plugin_exporter_set_section_property_type(void *plugin_handle, void *exporter_handle, int section_index, const af_wp_prop_type *property);
    typedef void af_wp_plugin_exporter_set_variable_property_type(void *plugin_handle, void *exporter_handle, int section_index, int variable_index, const af_wp_prop_type *property);
    typedef void af_wp_plugin_exporter_write_variable_data_block_type(void *plugin_handle, void *exporter_handle, int section_index, int variable_index, af_wp_data_type data_type, const void *data, int data_byte_size);
    typedef void af_wp_plugin_exporter_write_variable_data_row_type(void *plugin_handle, void *exporter_handle, int section_index, int nb_variables, const af_wp_value_type *value);
    typedef void af_wp_plugin_exporter_define_scope_type(void *plugin_handle, void *exporter_handle, const af_wp_prop_type *properties, int nb_properties);
    typedef void af_wp_plugin_exporter_upscope_type(void *plugin_handle, void *exporter_handle);

    typedef struct _af_wp_plugin_functions_type {

        int m_plugin_version;                                                           /**< Each time a plug-in is registered it's version is checked. It must correspond to AF_WAVEFORM_PARSER_PLUGIN_VERSION */
        void *m_plugin_handle;                                                          /**< The plugin handle must be non-NULL. It is passed as argument "plugin_handle" to all function requiring this argument. */

        /** This function must be implemented by the plugin. It is the first function to be called. It is called to retrieve plug-in properties. These plugin properties define how the plug-in is used. */
        af_wp_plugin_get_plugin_property_type                 *m_get_plugin_property;   
        af_wp_plugin_release_type                             *m_plugin_release;        /**< If this function is defined it is called just before un-registering the plug-in. */
        af_wp_plugin_get_error_message_type                   *m_get_error_message;     /**< If this function is defined it is called whenever another function reports an error to retrieve a human-readable error message. */
        

        /** This function must be implemented by the plugin. It is called to parse a file. It the function fails, other plug-ins and the built-in parser will attempt to parse the file.
            The file must not exist. If the plug-in handles the file type "*.db" and  BeSpice Wave Parser is asked to open the non-existing file "foo.db",
            the plug-in can handle that by allowing access to data currently held in memory or by managing a socket connection. */
        af_wp_plugin_parse_file_type                          *m_parse_file;            
        af_wp_plugin_reload_file_type                         *m_reload_file;           /**< This function must be implemented by the plugin if data base like access is possible. It is called to re-parse a file that has already been parsed but changed. */

        af_wp_plugin_release_file_type                        *m_release_file;          /**< If this function is defined it is called when BeSpice Wave Parser will not access any file for the specified file data any more. */
        af_wp_plugin_release_all_files_type                   *m_release_all_files;     /**< If this function is defined it is called when BeSpice Wave Parser will not access any file data any more. */
        af_wp_plugin_get_file_property_type                   *m_get_file_property;     /**< This function must be implemented by the plugin. It is called to retrieve properties for a file that was just parsed or reloaded. */

        af_wp_plugin_get_section_handle_type                  *m_get_section_handle;    /**< This function must be implemented by the plugin. It is called to retrieve a section handle that will allow to access this section. */
        af_wp_plugin_get_section_property_type                *m_get_section_property;  /**< This function must be implemented by the plugin. It is called to retrieve properties for a data section. */
        af_wp_plugin_set_section_property_type                *m_set_section_property;

        af_wp_plugin_get_variable_property_type               *m_get_variable_property;    /**< This function must be implemented by the plugin. It is called to retrieve properties for a curve. */
        af_wp_plugin_copy_variable_data_type                  *m_copy_variable_data;       /**< If implemented by the plug-in, this function must copy the plug-in's data to a vector passed by BeSpice Wave Parser. */
        af_wp_plugin_retrieve_variable_data_type              *m_retrieve_variable_data;   /**< If implemented by the plug-in, this function must return the plug-in's data to BeSpice Wave Parser where it will be copied. */

        af_wp_plugin_get_hierarchical_instance_property_type               *m_get_hierarchical_instance_property;              /**< This function is optional. It is used if the plugin uses hierarchical instances (scopes). */
        af_wp_plugin_get_hierarchical_instance_sub_instance_index_type     *m_get_hierarchical_instance_sub_instance_index;    /**< This function is optional. It is used if the plugin uses hierarchical instances (scopes). */
        af_wp_plugin_get_hierarchical_instance_variable_index_type         *m_get_hierarchical_instance_variable_index;        /**< This function is optional. It is used if the plugin uses hierarchical instances (scopes). */

        af_wp_plugin_variable_value_iterator_define_type      *m_variable_value_iterator_define;    /**< If implemented, the plug-in allows access to it's data using an iterator. The function must return a handle that will be used when retrieving the iterator's data. */
        af_wp_plugin_variable_value_iterator_release_type     *m_variable_value_iterator_release;   /**< This function releases the iterator after all data has been read. */
        af_wp_plugin_variable_value_iterator_read_next_type   *m_variable_value_iterator_read_next; /**< This function must copy the iterator's data to the passed data structures. Then the iterator is incremented. */

        af_wp_plugin_get_exporter_property_type               *m_get_exporter_property;
        af_wp_plugin_exporter_create_type                     *m_exporter_create;
        af_wp_plugin_exporter_release_type                    *m_exporter_release;
        af_wp_plugin_exporter_set_property_type               *m_exporter_set_property;
        af_wp_plugin_exporter_set_section_property_type       *m_exporter_set_section_property;
        af_wp_plugin_exporter_set_variable_property_type      *m_exporter_set_variable_property;
        af_wp_plugin_exporter_write_variable_data_block_type  *m_exporter_write_variable_data_block;
        af_wp_plugin_exporter_write_variable_data_row_type    *m_exporter_write_variable_data_row;
        af_wp_plugin_exporter_define_scope_type               *m_exporter_define_scope;
        af_wp_plugin_exporter_upscope_type                    *m_exporter_upscope;

    } af_wp_plugin_functions_type;


    /** \brief This function allows initialize a shared library to be used as data plug-in.

        It is the main plug-in access function. It creates an object of cCsvParserPlugin and initializes the structure af_wp_plugin_functions_type
        with pointers to the other library access functions.

        All shared libraries that will be registered automatically or by calling af_wp_register_plugin_library must implement this function.

    */
    AF_BSP_API af_return_code af_wp_plugin_initialize(af_wp_plugin_functions_type *functions);


#ifdef __cplusplus
}
#endif

/** @} */

#endif

