/*
    Copyright Analog Flavor 2013-2024

    All rights reserved.

    This software contains proprietary and confidential information which is the
    property of Analog Flavor and is subject to license terms.

    This file may be used in accordance with the license agreement 
    provided with the software.

    This copyright notice may not be removed or altered.
*/

/*! \file af_wp_types.h
  \brief This file contains the type definitions required by the BeSpice Wave Parser API.

  The BeSpice Wave Parser API allows to parse spice simulation results and to access the data that has been generated by the parser.

*/

#ifndef __af_wp_types__
#define __af_wp_types__

#include "af_return_codes.h"

#ifdef _MSC_VER /* visual studio compiler */

    typedef signed __int64       int64_t;
    typedef unsigned __int64     uint64_t;

#elif defined __AF_XCODE_FRAMEWORK__
#else

   #include <inttypes.h>

#endif

#ifdef __cplusplus
extern "C" {
#endif

    /** The all values and vectors can have several different data types. The returned values must be interpreted accordingly.

        Parameterized curves have individual x-axis values for each curve. Otherwise the x-axis data is defined
        in one or several sweep variables that are valid for all curves in the data section.

        Sparse curve define indices to determine their x-axis value. These indices allow to find the x-axis value
        in the sweep variable. The sweep variable is valid for all ordinary curves in the data section.
    */
    typedef enum {

        af_wp_data_error = 0, /**< An error occurred during a data type query. */
        af_wp_data_empty, /**< the curve has no data. */
        af_wp_data_string, /**< used by af_wp_get_section_property_data_type() if a section property is a string */
        af_wp_data_wide_string, /**< the member af_wp_value_type.m_string_value is a wide char string that has been casted to char*. This type may only be used by af_wp_curve_value_iterator_write_next. */
        af_wp_data_constant_size_string, /**< this type denotes a string vector where all entries have an identical size */
        af_wp_data_string_double, /**< used by af_wp_get_section_property_data_type() if a section property is a string and double */
        af_wp_data_wide_string_double, /**< used by af_wp_get_section_property_data_type() if a section property is a string and double. The member af_wp_value_type.m_string_value is a wide char string that has been casted to char*.  */
        af_wp_data_string_double_complex, /**< used by af_wp_get_section_property_data_type() if a section property is a string and double_complex */
        af_wp_data_string_int, /**< used by af_wp_get_section_property_data_type() if a section property is a string and integer */
        af_wp_data_float, /**< float data type */
        af_wp_data_double, /**< double data type */
        af_wp_data_float_complex, /**< float complex data type */
        af_wp_data_double_complex, /**< double complex data type */
        af_wp_data_int, /**< integer data type */
        af_wp_data_unsigned_int, /**< unsigned integer data type */
        af_wp_data_unsigned_int64, /**< 64 bit unsigned integer data type */
        af_wp_data_logic_vector, /**< logic data type */

        af_wp_data_float_parameterized_float, /**< parameterized float data type, float y values are given. The x-values are different for each curve of the data section */
        af_wp_data_double_parameterized_double, /**< parameterized double data type, double y values are given. The x-values are different for each curve of the data section. */
        af_wp_data_double_parameterized_float, /**< parameterized float data type, float y values and double x values are given. The x-values are different for each curve of the data section. */
        af_wp_data_float_parameterized_float_complex, /**< parameterized float complex data type, float complex y values are given. The x-values are different for each curve of the data section. */
        af_wp_data_double_parameterized_double_complex, /**< parameterized double complex data type, double complex y values are given. The x-values are different for each curve of the data section. */
        af_wp_data_double_parameterized_string, /**< parameterized string data type, string y values are given, the x-values are double values. They are different for each curve of the data section. */
        af_wp_data_double_parameterized_uint8_enum, /**< parameterized enumerated data type, uint8 y values are given. The double x-values are different for each curve of the data section. */
        af_wp_data_double_parameterized_int8, /**< parameterized 8 bit integer data type, double y values are given. The x-values are different for each curve of the data section. */
        af_wp_data_double_parameterized_int32, /**< parameterized 32 bit integer data type, double y values are given. The x-values are different for each curve of the data section. */
        af_wp_data_double_parameterized_uint8, /**< parameterized unsigned 8 bit integer data type, double y values are given. The x-values are different for each curve of the data section. */
        af_wp_data_double_parameterized_uint32, /**< parameterized unsigned 32 bit integer data type, double y values are given. The x-values are different for each curve of the data section. */
        af_wp_data_double_parameterized_uint64, /**< parameterized unsigned 64 bit integer data type, double y values are given. The x-values are different for each curve of the data section. */
        af_wp_data_double_parameterized_logic_vector,  /**< parameterized logic data type. The x-values are different for each curve of the data section. */

        af_wp_data_uint64_sparse_logic_vector, /**< sparse logic data type, one or several char y values are given. The x-value indices define the corresponding index in the sweep variable of the data section. The indices are given as uint64_t integer. They different for each curve of the data section. */

        af_wp_data_scalar_struct, /**< This defines a struct data type. The variable contains only 1 value of this type. See \link page_structured_data_types Structured Data Types \endlink*/
        af_wp_data_vector_struct, /**< This defines a struct data type. The variable contains 0 or more value of this type. See \link page_structured_data_types Structured Data Types \endlink*/

        af_wp_data_scalar_string, /**< Used by struct data type. The variable contains only 1 string value. */
        af_wp_data_scalar_double, /**< Used by struct data type. The variable contains only 1 double value. */
        af_wp_data_scalar_double_complex, /**< Used by struct data type. The variable contains only 1 double complex value. */
        af_wp_data_scalar_float, /**< Used by struct data type. The variable contains only 1 float value. */
        af_wp_data_scalar_float_complex, /**< Used by struct data type. The variable contains only 1 float complex value. */

        af_wp_data_last

    } af_wp_data_type;


    /** The units of the parsed curves are only available if specified in the parsed file. */
    typedef enum {

        af_wp_unit_error = 0, /**< An error occurred during a unit type query. */
        af_wp_unit_other, /**< The unit is of unknown type or a type that is only available as string. */
        af_wp_unit_seconds, /**< The curve values are time values given in seconds. */
        af_wp_unit_hours, /**< The curve values are time values given in hours. */
        af_wp_unit_hertz, /**< The curve values are frequencies given in hertz. */
        af_wp_unit_volt, /**< The curve values are voltages given in volt. */
        af_wp_unit_ampere, /**< The curve values are currents given in ampere. */
        af_wp_unit_time_stamp, /**< The curve values in second from 1/1/1970. */
        af_wp_unit_degree_centigrade, /**< The curve values is a temperature given in degree centigrade. */
        af_wp_unit_watt, /**< The curve values is a power given in watt. */
        af_wp_unit_none, /**< The curve values don't have a unit. */
        af_wp_unit_logic, /**< The curve values are logic values. */

        af_wp_unit_last

    } af_wp_unit_type;


    /** A variable might have no own values. Instead the values are derived from another variable by a simple function.
        This enum defines the available functions for this derivation. */
    typedef enum {

        af_wp_derivation_function_undefined = 0,
        af_wp_derivation_function_identity /**< The values in the derived curve are identical to the values in the original one. */

    } af_wp_derivation_function_type;



    /** Curves and sections can have properties.

        There are generic properties defined by their name and properties defined by their type.

        Properties are also used to define parser and data plugin options.

        Boolean values are identical to integer value where authorized values are 1 and 0.
    */
    typedef enum _af_wp_prop_enum_type {

        af_wp_prop_invalid = 0,  /**< The property is invalid. */

        af_wp_prop_string,  /**< Defines a generic property defined by it's name. The value is given as string. */
        af_wp_prop_integer,  /**< Defines a generic property defined by it's name. The value is given as integer. */
        af_wp_prop_double,  /**< Defines a generic property defined by it's name. The value is given as double. */
        af_wp_prop_complex,  /**< Defines a generic property defined by it's name. The value is given as complex. */

        af_wp_prop_file_name,  /**< The property defines a file name. The value is given as string. */
        af_wp_prop_file_id,  /**< The property defines an integer identifier. This identifier is unique for a given file name. */
        af_wp_prop_file_type,  /**< The property defines a file type. The value is given as string. */
        af_wp_prop_file_variant,  /**< The property defines a variant of a file type. The value is given as string. */
        af_wp_prop_file_data_base_access,  /**< The property indicates if the file can be accessed like a data base. The value is given as boolean value. */
        af_wp_prop_file_number_of_sections, /**< The property indicates how many data sections are available. The value is given as integer value. */
        af_wp_prop_file_changed, /**< The property indicates that a file changed. The value is given as boolean value. */
        af_wp_prop_file_closed, /**< The property indicates that a file was closed. The value is given as boolean value. */
        af_wp_prop_file_reload_in_thread,
        af_wp_prop_file_can_be_closed_by_user,
        af_wp_prop_temperature,

        af_wp_prop_global_export_file_name,  /**< The property defines the name of the file that created while exporting. The value is given as string. */
        af_wp_prop_global_number_of_types,  /**< The property indicates how many types are available. The value is given as integer value. */
        af_wp_prop_global_log_file_name,  /**< If this property is set debug messages will be printed to the specified file. The value is given as string. */
        af_wp_prop_global_current_working_directory, /**< This property makes sure, that relative paths are found correctly. The value is given as string. */

        af_wp_prop_global_release_date,  /**< This property holds the date when the current version of the library was built. The date is returned as an integer such as 170823 for the 23rd of August 2017. */
        af_wp_prop_global_release_version,  /**< This property holds the current version of the library as string. In addition to the build date it has some additional information that allows to precisely identify the version. */
        af_wp_prop_global_expiry_date,  /**< This property holds the date when builtin license for the current library expires. The date is returned as an integer such as 170823 for the 23rd of August 2017. */
        af_wp_prop_global_license_file_expiry_date,  /**< This property holds the date when license file for the current library expires. The date is returned as an integer such as 170823 for the 23rd of August 2017. */
        af_wp_prop_global_license_server_expiry_date,  /**< This property holds the date when license server for the current library expires. The date is returned as an integer such as 170823 for the 23rd of August 2017. */
        af_wp_prop_global_license_server_connection,  /**< This property holds the connection information for the license server as a string. */
        af_wp_prop_global_license_server_error_messages,  /**< This property holds debug information on license server as a string. */

        af_wp_prop_section_name,  /**< The property defines a section name. The value is given as string. */
        af_wp_prop_section_analysis_type,  /**< The property gives a section's analysis type. The value is given as string. */
        af_wp_prop_section_run_index,  /**< The property gives a section's run index. The value is given as integer. */
        af_wp_prop_section_string_run_parameter, /**< The property gives a section's run parameter. The value is given as string. The run parameter might also have a name. */
        af_wp_prop_section_integer_run_parameter, /**< The property gives a section's run parameter. The value is given as integer. The run parameter might also have a name. */
        af_wp_prop_section_double_run_parameter, /**< The property gives a section's run parameter. The value is given as double value. The run parameter might also have a name. */
        af_wp_prop_section_complex_run_parameter, /**< The property gives a section's run parameter. The value is given as complex value. The run parameter might also have a name. */
        af_wp_prop_section_x_start, /**< The property gives a section's first x-axis value. The value is given as double value. */
        af_wp_prop_section_x_step, /**< The property gives a section's x-axis value step. The value is given as double value. */
        af_wp_prop_section_x_stop, /**< The property gives a section's last x-axis value. The value is given as double value. */
        af_wp_prop_section_x_axis_unit, /**< The property gives a section's last x-axis unit. The value is given as af_wp_unit_type casted to an integer value or as string value. */
        af_wp_prop_section_number_of_sweep_variables, /**< The property gives the number of sweep variables in the section. The value is given as integer. */
        af_wp_prop_section_total_number_of_variables, /**< The property gives the total number of variables in the section. The value is given as integer. */
        af_wp_prop_section_changed, /**< The property indicates if a section changed. The value is given as boolean value. */
        af_wp_prop_section_alter_index,  /**< The property gives a section's alter index. The value is given as integer. */
        af_wp_prop_section_identifier,  /**< The property defines a section's unique identifier. The value determined by the waveform parser. It is given as integer. */
        af_wp_prop_section_number_of_instances, /**< The property gives the number of hierarchical instances in the section. The value is given as integer. */
        af_wp_prop_section_flag_used_for_link, /**< The property informs if the section is used for linking curves for example to a schematic viewer. The value is given as integer. */
        af_wp_prop_section_parent_section_id, /**< The property informs if the section has a parent section. This parent relation can be defined in a logFile. The value is given as integer. */
        af_wp_prop_section_vcd_dumpoff,
        af_wp_prop_section_vcd_dumpon,

        af_wp_prop_variable_name,  /**< The property defines a curve name. The value is given as string. */
        af_wp_prop_variable_x_axis_unit, /**< The property gives a curve's last x-axis unit. The value is given as af_wp_unit_type casted to an integer value or as string value. */
        af_wp_prop_variable_y_axis_unit, /**< The property gives a curve's last y-axis unit. The value is given as af_wp_unit_type casted to an integer value or as string value. */
        af_wp_prop_variable_sweep_variable, /**< The property indicates if a curve is a sweep variable. The value is given as boolean value. */
        af_wp_prop_variable_data_type, /**< The property gives a curve's data type. The value is given as af_wp_data_type casted to an integer value. */
        af_wp_prop_variable_number_of_values, /**< The property gives the number of values for a curves The value is given as integer. */
        af_wp_prop_variable_changed, /**< The property indicates if a curve changed. The value is given as boolean value. */
        af_wp_prop_variable_string_data_size, /**< If the curve has a string data type with a constant string length, this property indicates this length. */
        af_wp_prop_variable_scale, /**< If a scaling factor applies to the curve's data, this property indicates this factor. It is given as double value. */
        af_wp_prop_variable_plot_color, /**< If the curve should be plotted using a particular color, this property indicates this color. It is given as string value. */
        af_wp_prop_variable_plot_line_style, /**< If the curve should be plotted using a particular line style, this property indicates this line style. It is given as string value. */
        af_wp_prop_variable_plot_symbol, /**< If the curve should be plotted using a particular symbol, this property indicates this symbol. It is given as string value. */
        af_wp_prop_variable_min_x_value, /**< This property gives the curve's minimum x-axis value. It is only available after having been computed. It is given as double value. */
        af_wp_prop_variable_max_x_value,/**< This property gives the curve's maximum x-axis value. It is only available after having been computed. It is given as double value. */
        af_wp_prop_variable_real_min_y_value, /**< This property gives the curve's minimum y-axis value. It is only available after having been computed. It is given as double value. */
        af_wp_prop_variable_real_max_y_value, /**< This property gives the curve's maximum y-axis value. It is only available after having been computed. It is given as double value. */
        af_wp_prop_variable_imag_min_y_value, /**< This property gives the curve's minimum imaginary y-axis value. It is only available form complex curves after having been computed. It is given as double value. */
        af_wp_prop_variable_imag_max_y_value, /**< This property gives the curve's maximum imaginary y-axis value. It is only available form complex curves after having been computed. It is given as double value. */
        af_wp_prop_variable_x_axis_type_index, /**< The property gives a curve's x-axis type index. This index is an integer. It can be used to retrieve information about the used type. This property is only given for parameterized curves. */
        af_wp_prop_variable_y_axis_type_index, /**< The property gives a curve's y-axis type index. This index is an integer. It can be used to retrieve information about the used type. */
        af_wp_prop_variable_model, /**< The property gives a curve's model. The value is given as string. This property is mainly used for operating point analysis results. Example : "model = resistor".*/
        af_wp_prop_variable_region, /**< The property gives a curve's region. The value is given as string. This property is mainly used for operating point analysis results. Example : "region = saturation" or "region = linear".*/
        af_wp_prop_variable_assert, /**< The property gives a curve's assert. The value is given as string. This property is mainly used for safe operating area results.*/
        af_wp_prop_variable_instance, /**< The property gives a curve's instance. The value is given as string. This property is mainly used for safe operating area results.*/
        af_wp_prop_variable_expression, /**< The property gives a curve's expression. The value is given as string. This property is mainly used for safe operating area results.*/
        af_wp_prop_variable_severity, /**< The property gives a curve's severity. The value is given as string. This property is mainly used for safe operating area results. Example : "severity = warning".*/
        af_wp_prop_variable_status, /**< The property gives a curve's status. The value is given as string. This property is mainly used for safe operating area results. Example : "status = failed".*/
        af_wp_prop_variable_vcd_type, /**< Type of variable as defined in a vcd file. */
        af_wp_prop_variable_hierachical_instance_index, /**< The property gives a curve's hierarchical instance index as integer. */
        af_wp_prop_variable_left_bit_number, /**< The property defines the left bit number of a vector as integer. */
        af_wp_prop_variable_right_bit_number, /**< The property defines the right bit number of a vector as integer. */
        af_wp_prop_variable_derived_from_curve_index,  /**< The property defines another curve index as integer. This curve's values are defined as a function of the other's values. */
        af_wp_prop_variable_derived_from_curve_function, /**< This property defines the derivation function as integer. Therefor the enum type af_wp_derivation_function_type is cated into an interger value. */
        af_wp_prop_variable_equivalent_parent_net_name, /**< This property is added if the corresponding curve has been created with af_wp_create_equivalent_nets(). */
        af_wp_prop_calculator_expression_string, /**< If a curve was generated by the calculator, this property holds the evaluated expression as a string. */

        af_wp_prop_instance_name, /**< This property gives the name of an instances. It is given as string value. */
        af_wp_prop_instance_full_hierarchical_name, /**< This property gives full hierarchical the name of an instances. It is given as string value. */
        af_wp_prop_instance_number_of_sub_instances, /**< This property gives the number of sub-instances defined in a given instance. It is given as integer value. */
        af_wp_prop_instance_number_of_variables, /**< This property gives the number of variables defined in a given instance. It is given as integer value. */
        af_wp_prop_instance_hierarchical_level, /**< This property gives the hierarchical level of an instances. It is given as integer value. */
        af_wp_prop_instance_parent_instance_index, /**< This property gives the index of the parent instance of an instances. It is given as integer value. */
        af_wp_prop_instance_component_curve_index, /**< This property gives the index of the curve that is expanded in this instance. This requires the "af_wp_prop_parser_option_create_instance_for_expanded_curves" option.*/
        af_wp_prop_instance_vcd_type, /**< This property gives the type of an instances. It is given as string value. */
        af_wp_prop_instance_separator, /**< This property gives the separator used in the name of a sub-instances or hierarchical curve names. It is given as string value. */
        af_wp_prop_instance_subcircuit_name, /**< This property gives the name of the subcircuit in the spice simulation. This property might not be informed. */

        af_wp_prop_type_data_type, /**< This property gives the data type of a variable type. The value is given as af_wp_data_type casted to an integer value. */
        af_wp_prop_type_unit, /**< The property gives a type's unit. The value is given as af_wp_unit_type casted to an integer value or as string value. */
        af_wp_prop_type_number_of_sub_types, /**< This property gives the number of subtypes defined for a given type. It is given as integer value. The value is different from zero only for af_wp_data_scalar_struct and af_wp_data_vector_struct types. */
        af_wp_prop_type_description, /**< This property defines a description for the type. In general it is only used for the fields of a structured type. */
        af_wp_prop_type_name, /**< This property gives the name of a type. For structured data type it is the field name. */
        af_wp_prop_type_enumerated_value, /**< This property gives the string values defined for an enumerated data type. The property field m_integer_value holds the integer value corresponding to the enumerated value. This property also exists for some logic types. */
        af_wp_prop_type_string_data_size, /**< If the type defines a string data type with a constant string length, this property indicates this length. */

        af_wp_prop_parser_option_generate_sweep_variable, /**< Use this property to generate a sweep variable even if it is not available in the parsed file. The value is given as boolean value. */
        af_wp_prop_parser_option_convert_int_to_real, /**< Use this property to generate real variables instead of integer variables. The value is given as boolean value. */
        af_wp_prop_parser_option_generate_instance_hierarchy, /**< Use this property to make sure that the hierarchy of subcircuit instances is computed. */
        af_wp_prop_parser_option_parse_referenced_files, /**< If this property is set files referenced for example in a "logFile" are parsed automatically. */
        af_wp_prop_parser_option_release_curve_after_iterator_release, /**< If this property is set, the memory deallocation is simplified. Curves do no longer have to be released after an iterator has been released. */
        af_wp_prop_parser_option_generate_expanded_curves, /**< If this property is set, all curves based on bus types and structured types are also available as individual curves. This only applies to curves with sweep variable. */
        af_wp_prop_parser_option_generate_expanded_scalar_curves, /**< If this property is set, all curves based on structured types are also available as individual curves. This only applies to dc, op and soa analysis results and logfiles. */
        af_wp_prop_parser_option_generate_expanded_logic_bus_curves, /**< If this property is set, all curves based on logic bus types are also available as individual curves for each bit. */
        af_wp_prop_parser_option_create_instance_for_expanded_curves, /**< If this property is set, an instance is created for all curves based on bus types and structured types that are expanded. */
        af_wp_prop_parser_option_logic_level_0, /**< This property defines the real value attributed to logic 0 levels when the curve is transformed to a real-valued curve. */
        af_wp_prop_parser_option_logic_level_1, /**< This property defines the real value attributed to logic 1 levels when the curve is transformed to a real-valued curve. */
        af_wp_prop_parser_option_logic_level_X, /**< This property defines the real value attributed to logic X levels when the curve is transformed to a real-valued curve. */
        af_wp_prop_parser_option_logic_level_Z, /**< This property defines the real value attributed to logic Z levels when the curve is transformed to a real-valued curve. */
        af_wp_prop_parser_option_logic_level_U, /**< This property defines the real value attributed to logic U levels when the curve is transformed to a real-valued curve. */
        af_wp_prop_parser_option_logic_rise_fall_time, /**< This property defines the rise and fall time used when a logic curve is transformed to a real-valued curve. This value is also used for enumerated curves. */
        af_wp_prop_parser_option_enumerated_level, /**< This property defines the real value attributed to the enumerated curve value defined in m_string_value when the curve is transformed to a real-valued curve. The real attribute must be given in m_real_value. */
        af_wp_prop_parser_option_create_all_section_properties, /**< If this option is set, all parsed section properties are available over the interface. */
        af_wp_prop_parser_option_minimum_size_for_db_access,  /**< If this > 0, smaller variables will be loaded immediately into memory. This isn't supported for all parsers. */
        af_wp_prop_parser_option_read_all_properties,  /**< If this 1, additional properties (metadata) will be read when parsing a file. */
        af_wp_prop_parser_option_dont_read_data,  /**< If this 1, data will not be read when parsing a file. This isn't supported by all file types. */
        af_wp_prop_parser_option_simplify_instance_names_for_currents,  /**< Some simulators add the device type "v" or "c" to the curve names for current measuremants. As a consequence the curve names are incompatible with voltages. If this option is set, the device type is removed from the curve name. */
        af_wp_prop_parser_option_check_x_axis_values_for_multiple_runs, /**< Some file types might hold multiple simulation runs. Thererfore the data can stored in a single file or file section. This automatically detected. This option allows to disable the detection. */
        af_wp_prop_parser_option_first_x_axis_value, /**< Some file types can be read only partially on an x-axis interval. Therefore the first and the last x-axis values must be defined. */
        af_wp_prop_parser_option_last_x_axis_value, /**< Some file types can be read only partially on an x-axis interval. Therefore the first and the last x-axis values must be defined. */
        af_wp_prop_parser_option_parse_hidden_curves, /**< Some file formats define curves that aren't of much interest for most applications. If this property is set these curves are read as well. The default value is false. */

        af_wp_prop_parser_option_spice_read_included_files,  /**< If set to 1, the function af_wp_create_equivalent_nets() read all files included by the spice netlist. The default is 1. */

        af_wp_prop_parser_option_export_start,
        af_wp_prop_parser_option_export_stop,
        af_wp_prop_parser_option_export_ascii_precision,
        af_wp_prop_parser_option_export_vcd_time_scale,
        af_wp_prop_parser_option_export_split_complex_curves,
        af_wp_prop_parser_option_export_creation_date,
        af_wp_prop_parser_option_export_matrix_variable_name,
        af_wp_prop_parser_option_export_column_vector_variable_name,
        af_wp_prop_parser_option_export_unit_vector_variable_name,
        af_wp_prop_parser_option_export_compress,
        af_wp_prop_parser_option_export_double_precision,
        af_wp_prop_parser_option_export_verbose_level,
        af_wp_prop_parser_option_export_maximum_hierarchy_level,
        af_wp_prop_parser_option_export_instance_name,
        af_wp_prop_parser_option_export_string_options,
        af_wp_prop_parser_option_export_generate_instances,
        af_wp_prop_parser_option_export_radix_digital_curves,
        af_wp_prop_parser_option_export_interpolate_missing_values,
        af_wp_prop_parser_option_export_logic_low_real_value,
        af_wp_prop_parser_option_export_logic_high_real_value,
        af_wp_prop_parser_option_export_fixed_x_axis_value,

        af_wp_prop_plugin_name, /**< The property defines a plugin name. The value is given as string. */
        af_wp_prop_plugin_handled_extension, /**< The property defines the extensions handled by a plugin. The value is given as string. A plugin can handle several extensions. */
        af_wp_prop_plugin_version,

        af_wp_prop_plugin_exporter_name,
        af_wp_prop_plugin_exporter_message,
        af_wp_prop_plugin_exporter_option_complex_supported,
        af_wp_prop_plugin_exporter_option_strings_supported,
        af_wp_prop_plugin_exporter_option_integer_supported,
        af_wp_prop_plugin_exporter_option_logic_supported,
        af_wp_prop_plugin_exporter_option_multiple_dimensions_supported,
        af_wp_prop_plugin_exporter_option_additional_string_argument,

        af_wp_prop_last

    } af_wp_prop_enum_type;


    /** Curves and sections can have properties.

        The C struct af_wp_prop_type is used to set properties or to retrieve their values.
    */
    typedef struct _af_wp_prop_type {

        af_wp_prop_enum_type            m_type; /**< This struct member gives the type of the property. */
        const char                      *m_name; /**< This struct member defines the name of the property for generic properties. It might also be used by other properties such as af_wp_prop_section_string_run_parameter. */
        af_wp_data_type                 m_data_type; /**< This struct member gives the data type of the property. The Allowed types are af_wp_data_string, af_wp_data_double, af_wp_data_double_complex and af_wp_data_int. */
        const char                      *m_string_value; /**< This struct member gives the string value of the property if the data type is af_wp_data_string. */
        double                          m_real_value; /**< This struct member gives the real value of the property if the data type is af_wp_data_double. */
        double                          m_imag_value; /**< This struct member gives the imaginary part of the property's value if the data type is af_wp_data_double_complex. */
        int64_t                         m_integer_value; /**< This struct member gives the integer value of the property if the data type is af_wp_data_int. */

    } af_wp_prop_type;

    /** Curves define curve values.

        The C struct af_wp_value_type is used to read or write curve values values.
    */
    typedef struct _af_wp_value_type {

        af_wp_data_type                 m_data_type;  /**< This struct member gives the data type of the read or written value. When writing, the data type must coincide with the data type of the curve. */
        const char                      *m_string_value; /**< This struct member is used to read or write a string value. */
        double                          m_real_value; /**< This struct member is used to read or write a real value or the real part of a complex value. */
        double                          m_imag_value; /**< This struct member is used to read or write the imaginary part of a complex value. */
        int64_t                         m_integer_value; /**< This struct member is used to read or write an integer value. */
        uint64_t                        m_unsigned_integer_value; /**< This struct member gives the integer value of the property if the data type is af_wp_data_int. */

    } af_wp_value_type;


#ifdef __cplusplus
}
#endif


#endif

