/*
    Copyright Analog Flavor 2013-2024

    All rights reserved.

    This software contains proprietary and confidential information which is the
    property of Analog Flavor and is subject to license terms.

    This file may be used in accordance with the license agreement 
    provided with the software.

    This copyright notice may not be removed or altered.
*/

#ifndef __bspWaveWrapper__
#define __bspWaveWrapper__

#include <list>
#include <vector>
#include <string>

/// The template class bspWaveWrapper<>
///     simplifies the usage of the text-based command. The command “open_file <file_name>” can
///     be emitted by calling the method bspWaveWrapper::cmdOpenFileStd(const std::string &file_name).
///
/// The interface implementation such as "qtBspWaveWrapperImpl" must be specified in the template
///     specialization.
///
template <typename IMPL_T>
class bspWaveWrapper : public IMPL_T {

public:

    bspWaveWrapper() {
    }
    template <typename PARENT_T>
    bspWaveWrapper(PARENT_T parent) : IMPL_T(parent) {
    }
    template <typename PARENT_T>
    bspWaveWrapper(PARENT_T parent, int style_flags) : IMPL_T(parent, style_flags) {
    }
    template <typename PARENT_T>
    bspWaveWrapper(PARENT_T parent, int x, int y, int w, int h, int style_flags) : IMPL_T(parent, x, y, w, h, style_flags) {
    }

    /// This function sends a text command to BeSpice Wave and returns the return code.
    af_return_code cmdStd(const std::string &cmd) {
        return IMPL_T::cmdStd(cmd);
    }
    /// This function sends a text command to BeSpice Wave and returns the return code and the result of the query as string.
    af_return_code cmdStd(const std::string &cmd, std::string &string_result);
    /// This function sends a text command to BeSpice Wave and returns the return code and the result of the query as vector of strings.
    af_return_code cmdStd(const std::string &cmd, std::vector<std::string> &string_results);
    /// This function sends a text command to BeSpice Wave and returns the return code and the result of the query as integer.
    af_return_code cmdStd(const std::string &cmd, int &integer_result);

    /// The purpose of this command is to verify the current status of the commend execution queue. The returned value is "af_code_queued", when BeSpice is currently executing a command, "af_code_error" if the previous command execution failed and "af_code_ok" otherwise. 
    af_return_code cmdGetStatus();
    af_return_code cmdShowControlPanel();
    af_return_code cmdShowCommandLogger();
    af_return_code cmdHidePanels();
    af_return_code cmdShowFileOpenDialog();
    af_return_code cmdWidgetParentsChanged();
    af_return_code cmdCreateAnalogPlot(const std::string &plot_name, bool flag_new_tab = true, bool flag_switch_to_tab = false);
    af_return_code cmdCreateDigitalPlot(const std::string &plot_name, bool flag_switch_to_tab = false);
    af_return_code cmdClosePlot(const std::string &plot_name);
    /// This command shows the export as data dialog. 
    af_return_code cmdShowExportAsDataDialog(const std::string &plot_name);
    /// This command shows the export as image dialog. 
    af_return_code cmdShowExportAsImageDialog(const std::string &plot_name);
    af_return_code cmdBringWindowToFront();

    ///  This command opens the file <file_name> and parses it. 
    af_return_code cmdOpenFileStd(const std::string &file_name);
    af_return_code cmdOpenFilesStd(const std::list<std::string> &files);
    /// This command reloads the file <file_name> and parses it. All shown curves are updated automatically. 
    af_return_code cmdReloadFileStd(const std::string &file_name);
    /// This command closes the file <file_name> and all associated sections and curves. 
    af_return_code cmdCloseFileStd(const std::string &file_name);
    af_return_code cmdShowFileInTextEditor(const std::string &file);

    /// This command instructs BeSpice to execute the commands listed in the text file <file_name>. The commands in this file must be separated by newline characters. A command can't be split into several lines. 
    af_return_code cmdExecuteCommandFileStd(const std::string &file_name);

    af_return_code cmdCreatePlotIfEmpty();
    af_return_code cmdUnzoomPlot(const std::string &plot_name);
    af_return_code cmdClearPlot(const std::string &plot_name);
    af_return_code cmdAddCurveToPlot(const std::string &plot_name, const std::string &section_name, const std::string &curve_name, bool flag_clear);
    af_return_code cmdAddCurveToPlotByIdAndIndex(const std::string &plot_name, size_t section_id, size_t curve_index, bool flag_clear);
    af_return_code cmdAddCurveToPlotByIndex(const std::string &plot_name, const std::string &section_name, size_t curve_index, bool flag_clear);
    af_return_code cmdAddCurveToPlotByIndex(const std::string &plot_name, const std::string &section_name, size_t curve_index, const std::vector<size_t> &sweep_perm, const std::vector<size_t> &sweep_indices, bool flag_clear);
    af_return_code cmdRemoveCurveFromPlotByIndex(const std::string &plot_name, const std::string &section_name, size_t curve_index);
    af_return_code cmdRemoveCurveFromPlotByIndex(const std::string &plot_name, const std::string &section_name, size_t curve_index, const std::vector<size_t> &sweep_perm, const std::vector<size_t> &sweep_indices);
    af_return_code cmdHighlightCurveInPlotByIndex(const std::string &plot_name, const std::string &section_name, size_t curve_index, bool flag_clear);
    af_return_code cmdHighlightCurveInPlotByIndex(const std::string &plot_name, const std::string &section_name, size_t curve_index, const std::vector<size_t> &sweep_perm, const std::vector<size_t> &sweep_indices, bool flag_clear);
    af_return_code cmdHighlightCurveInPlotByIdAndIndex(const std::string &plot_name, size_t section_id, size_t curve_index, bool flag_clear);
    af_return_code cmdUnhighlightCurveInPlotByIndex(const std::string &plot_name, const std::string &section_name, size_t curve_index, bool flag_clear);
    af_return_code cmdUnhighlightCurveInPlotByIndex(const std::string &plot_name, const std::string &section_name, size_t curve_index, const std::vector<size_t> &sweep_perm, const std::vector<size_t> &sweep_indices, bool flag_clear);
    af_return_code cmdGetNbOfSections(int &nb_sections);
    af_return_code cmdGetSectionName(int section_index, std::string &section_name);
    af_return_code cmdGetSectionFileName(const std::string &section_name, std::string &file_name);
    af_return_code cmdAddCurveAnnotationToPlotByIdAndIndex(const std::string &plot_name, size_t section_id, size_t curve_index, const std::string &annotation_text, double x_coordinate);
    af_return_code cmdClearAnnotationsInPlot(const std::string &plot_name);
    af_return_code cmdStatusBarMessage(size_t severity, const std::string &msg);
    af_return_code cmdLoggerMessage(size_t severity, const std::string &msg);
    af_return_code cmdStatusBarProgress(size_t progress);
    af_return_code cmdDetachTab(const std::string &tab_name, bool flag_detach);
    af_return_code cmdShowToolbarInTab(const std::string &tab_name, bool flag_show);

    af_return_code cmdSetOptionShowExitButton(bool flag_show);
    af_return_code cmdSetOptionShowDetachButton(bool flag_show);
    af_return_code cmdSetOptionShowAttachButton(bool flag_show);
    af_return_code cmdSetOptionUseWhiteBackground(bool flag_white);
    af_return_code cmdSetOptionShowDigitalPlotButton(bool flag_show);
    af_return_code cmdSetOptionToolBarStyle(const std::string &style) { return cmdStd(std::string("set_option tool_bar_style " + style)); }
    // style must be either none or top_left
    af_return_code cmdSetOptionPlotWindowStyle(const std::string &plot_name, const std::string &style) { return cmdStd(std::string("set_option plot_window_button_style \"" + plot_name + "\" " + style)); }

    af_return_code cmdGetWindowSizeHints(int &width, int &height);

    // the following functions are identical to menu / toolbar / keyboard shortcut actions
    //      if no "plot_name" is given, the command applies to the currently active plot window
    af_return_code cmdCopyCurve(const std::string &plot_name = "") { return cmdStd(std::string("copy_curve \"" + plot_name + "\"")); }
    af_return_code cmdPasteCurve(const std::string &plot_name = "") { return cmdStd(std::string("paste_curve \"" + plot_name + "\"")); }
    af_return_code cmdCutCurve(const std::string &plot_name = "") { return cmdStd(std::string("cut_curve \"" + plot_name + "\"")); }
    af_return_code cmdRemoveCurve(const std::string &plot_name = "") { return cmdStd(std::string("remove_curve \"" + plot_name + "\"")); }
    af_return_code cmdSelectAllCurves(const std::string &plot_name = "") { return cmdStd(std::string("select_all_curves \"" + plot_name + "\"")); }
    af_return_code cmdUndo(const std::string &plot_name = "") { return cmdStd(std::string("undo \"" + plot_name + "\"")); }
    af_return_code cmdRedo(const std::string &plot_name = "") { return cmdStd(std::string("redo \"" + plot_name + "\"")); }
    af_return_code cmdMoveLeft(const std::string &plot_name = "") { return cmdStd(std::string("move_left \"" + plot_name + "\"")); }
    af_return_code cmdMoveRight(const std::string &plot_name = "") { return cmdStd(std::string("move_right \"" + plot_name + "\"")); }
    af_return_code cmdMoveUp(const std::string &plot_name = "") { return cmdStd(std::string("move_up \"" + plot_name + "\"")); }
    af_return_code cmdMoveDown(const std::string &plot_name = "") { return cmdStd(std::string("move_down \"" + plot_name + "\"")); }
    af_return_code cmdMovePageUp(const std::string &plot_name = "") { return cmdStd(std::string("move_page_up \"" + plot_name + "\"")); }
    af_return_code cmdMovePageDown(const std::string &plot_name = "") { return cmdStd(std::string("move_page_down \"" + plot_name + "\"")); }
    af_return_code cmdMoveEnd(const std::string &plot_name = "") { return cmdStd(std::string("move_end \"" + plot_name + "\"")); }
    af_return_code cmdMoveHome(const std::string &plot_name = "") { return cmdStd(std::string("move_home \"" + plot_name + "\"")); }
    af_return_code cmdZoomX(const std::string &plot_name = "") { return cmdStd(std::string("zoom_x \"" + plot_name + "\"")); }
    af_return_code cmdZoomY(const std::string &plot_name = "") { return cmdStd(std::string("zoom_y \"" + plot_name + "\"")); }
    af_return_code cmdZoomIn(const std::string &plot_name = "") { return cmdStd(std::string("zoom_in \"" + plot_name + "\"")); }
    af_return_code cmdZoomOut(const std::string &plot_name = "") { return cmdStd(std::string("zoom_out \"" + plot_name + "\"")); }
    af_return_code cmdUnzoom(const std::string &plot_name = "") { return cmdStd(std::string("unzoom \"" + plot_name + "\"")); }
    af_return_code cmdPreviousViewPort(const std::string &plot_name = "") { return cmdStd(std::string("previous_view_port \"" + plot_name + "\"")); }
    af_return_code cmdNextViewPort(const std::string &plot_name = "") { return cmdStd(std::string("next_view_port \"" + plot_name + "\"")); }
    af_return_code cmdEnableCursorMode(const std::string &plot_name = "") { return cmdStd(std::string("enable_cursor_mode \"" + plot_name + "\"")); }
    af_return_code cmdDisableCursorMode(const std::string &plot_name = "") { return cmdStd(std::string("disable_cursor_mode \"" + plot_name + "\"")); }
    af_return_code cmdToogleCursorMode(const std::string &plot_name = "") { return cmdStd(std::string("toggle_cursor_mode \"" + plot_name + "\"")); }
    af_return_code cmdClearCursors(const std::string &plot_name = "") { return cmdStd(std::string("clear_cursors \"" + plot_name + "\"")); }

    af_return_code checkInstanceIdentifier(size_t id);

    std::string convertVectorToString(const std::vector<size_t> &vec);
    std::string replaceDoubleQuotes(const std::string &original_txt) const;
    void splitString(const std::string &txt, char separator, std::vector<std::string> &tokens) const;
    void replaceInString(std::string &s, const std::string &old_str, const std::string &new_str) const;
    static void toLowerCase(std::string &s);

    static std::string convertReturnCodeToString(af_return_code rc);

};


#endif // __bspWaveWrapper__
