/*
    Copyright Analog Flavor 2013-2024

    All rights reserved.

    This software contains proprietary and confidential information which is the
    property of Analog Flavor and is subject to license terms.

    This file may be used in accordance with the license agreement 
    provided with the software.

    This copyright notice may not be removed or altered.
*/

#ifndef __bspwave_widget__h__
#define __bspwave_widget__h__

#include "af_return_codes.h"

#ifdef __cplusplus
extern "C" {
#endif

    #define AF_BSPWAVE_STYLE_DEBUG 1

    #define AF_BSPWAVE_STYLE_USE_TOOLBAR 2
    #define AF_BSPWAVE_STYLE_USE_NOTEBOOK 4
    #define AF_BSPWAVE_STYLE_USE_AUTO_SIDEBAR 8
    #define AF_BSPWAVE_STYLE_USE_MENUBAR 16

    #define AF_BSPWAVE_STYLE_USE_INTEGRATOR_LICENSE_TEST 32

    #define AF_BSPWAVE_STYLE_AUTO_LOAD_SAVE_CONFIG 64

    typedef enum {

        af_bspwave_notification_undefined,

        af_bspwave_notification_close_button_activated,
        af_bspwave_notification_attach_button_activated,
        af_bspwave_notification_detach_button_activated,
        af_bspwave_notification_open_file_button_activated,

        af_bspwave_notification_focus_in,
        af_bspwave_notification_focus_out,

        af_bspwave_notification_logger_message, /**< The message defined by m_string_value was sent to the message logger. The severity is given by m_integer_value. */
        af_bspwave_notification_status_bar_message, /**< The message defined by m_string_value was sent to the status bar. The severity is given by m_integer_value. */
        af_bspwave_notification_progress_message,  /**< The message defined by m_string_value was sent to the progress bar. The progress is given by m_integer_value ( 0 .. 100). */

        af_bspwave_notification_file_opened, /**< The file defined by m_string_value was opened. */
        af_bspwave_notification_file_closed, /**< The file defined by m_string_value was closed. */
        af_bspwave_notification_all_files_closed, /**< All open file were closed. */
        af_bspwave_notification_section_used_for_link_changed,  /**< The user selected another section for linking for example to schematic. */

        af_bspwave_notification_configuration_changed, /** The configuration changed (read from file or user action). */

        /** these notification are specific to the integrating application */
        af_bspwave_notification_integration_specific_action_1,
        af_bspwave_notification_curve_to_schematic_mapping_defined,
        af_bspwave_notification_copy_highlighting

    } af_bspwave_callback_notification_enum_type;

    typedef struct _af_bspwave_callback_notification_type {

        af_bspwave_callback_notification_enum_type      m_type;
        const char                                      *m_string_value;
        const char                                      *m_second_string_value;
        int                                             m_integer_value;

    } af_bspwave_callback_notification_type;

    typedef struct _af_bspwave_callback_type {

        void *callback_object;

        /* 1st argument : the above callback_object */
        void (*widget_destroyed)(void *obj);
        /* 1st argument : the above callback_object
           2nd argument : section name
           3rd argument : curve name
           4th argument : flag clear highlighting for all other highlighted curves
           5th argument : flag, true if from browser */
        void (*curve_highlighted)(void *obj, const char *section_name, const char *curve_name, int section_id, int curve_index, int flag_clear, int from_browser);
        void(*curve_un_highlighted)( void *obj, const char *section_name, const char *curve_name, int section_id, int curve_index, int flag_clear, int from_browser );
        void(*curve_activated)( void *obj, const char *section_name, const char *curve_name, int section_id, int curve_index, int flag_clear, int from_browser );

        /* BeSpice Wave received an incompatible text drop.
           This callback function allows to handle the drop externally. 
           The function can return 0. In this case BeSpice Wave tries to handle the drop by itself.
           Otherwise the functon can return a modified string that will be handled by BeSpice Wave.
           Returning an empty string will prevent all further actions. */
        const char* (*drop_received)(void *obj, const char *plot_name, const char *received_text);

        /* This callback function allows to change the string dragged from BeSpice Wave
           to another application. Therefore the xml string must be parsed and a corresponding
           text string must be returned.
           This function is ignored if it returns 0. */
        const char* (*generate_drag_string)(void *obj, const char *xml_string);

        /* This callback function is called each time the current cursor value change. */
        void (*cursor_values_changed)(void *obj, const char *plot_name, const char* const* cursor_values);

        /* Depending on your licence it might be necessary to implement this function to run BeSpice Wave widget
           1st argument : the above callback_object.
           (warning) this function might be called from a thread ! */
        void (*check_integrator_license)(void *obj, void *data);

        int (*notify)(void *obj, const af_bspwave_callback_notification_type *description);

    } af_bspwave_callback_type;


    typedef enum _af_bspwave_plugin_property_enum_type {

        af_bspwave_plugin_invalid_property = 0,

        af_bspwave_plugin_get_name,
        af_bspwave_plugin_get_platform,
        af_bspwave_plugin_get_target,
        af_bspwave_plugin_get_nb_widgets,
        af_bspwave_plugin_get_widget_name,

        af_bspwave_plugin_widget_get_flag_multiple_pages_allowed,
        af_bspwave_plugin_widget_get_flag_is_graphical_page,
        af_bspwave_plugin_widget_get_nb_plot_windows,
        af_bspwave_plugin_widget_get_max_nb_of_curves,
        af_bspwave_plugin_widget_get_allowed_curve_data_type_x,
        af_bspwave_plugin_widget_get_allowed_curve_data_type_y,
        af_bspwave_plugin_widget_get_page_parameter,

        af_bspwave_plugin_widget_set_will_close,
        af_bspwave_plugin_widget_set_has_been_attached,
        af_bspwave_plugin_widget_set_has_been_detached,


        af_bspwave_plugin_widget_action_set_plot_name,
        af_bspwave_plugin_widget_action_plot_is_shown,
        af_bspwave_plugin_widget_action_plot_is_hidden,
        af_bspwave_plugin_widget_action_curve_was_added,
        af_bspwave_plugin_widget_action_curve_was_removed,
        af_bspwave_plugin_widget_action_set_page_parameter

    } af_bspwave_plugin_property_enum_type;


    typedef enum _af_bspwave_plugin_data_type {

        af_bspwave_plugin_data_string,
        af_bspwave_plugin_data_integer,
        af_bspwave_plugin_data_integer_integer

    } af_bspwave_plugin_data_type;



    typedef struct _af_bspwave_plugin_property_type {

        af_bspwave_plugin_property_enum_type            m_type;
        const char                                      *m_name;
        af_bspwave_plugin_data_type                     m_data_type;
        const char                                      *m_string_value;
        int                                             m_integer_value;
        int                                             m_integer_value_2;

    } af_bspwave_plugin_property_type;

    #define AF_BSPWAVE_WIDGET_INFO_TYPE_BROWSER_HANDLE 1
    #define AF_BSPWAVE_WIDGET_INFO_TYPE_TOOLBAR_HANDLE 2
    #define AF_BSPWAVE_WIDGET_INFO_TYPE_TOOLBAR_BEST_SIZE 3
    #define AF_BSPWAVE_WIDGET_INFO_TYPE_MENUBAR_HANDLE 4
    #define AF_BSPWAVE_WIDGET_INFO_TYPE_CHECK_MENU_ITEM_ENABLED 5
    #define AF_BSPWAVE_WIDGET_INFO_TYPE_EXECUTE_MENU_COMMAND 6
    #define AF_BSPWAVE_WIDGET_INFO_TYPE_TAB_CTRL_HANDLE 7
    #define AF_BSPWAVE_WIDGET_INFO_TYPE_MEASUREMENTS_HANDLE 8
    #define AF_BSPWAVE_WIDGET_INFO_TYPE_PAGE_PARAMETERS_HANDLE 9
    #define AF_BSPWAVE_WIDGET_INFO_TYPE_LAYOUT_HANDLE 10
    #define AF_BSPWAVE_WIDGET_INFO_TYPE_SIDEBAR_HANDLE 11


#ifdef __cplusplus
}
#endif

#endif /*__bspwave_widget__h__*/
