/*
    Copyright Analog Flavor 2013-2024

    All rights reserved.

    This software contains proprietary and confidential information which is the
    property of Analog Flavor and is subject to license terms.

    This file may be used in accordance with the license agreement 
    provided with the software.

    This copyright notice may not be removed or altered.
*/


#ifndef __winBspWaveWrapperRuntime__
#define __winBspWaveWrapperRuntime__

#include <windows.h>

#include "bspwave_win_widget.h"
#include "bspWaveWrapper.h"

#include <string>
#include <list>

/* 
    The class winBspWaveWrapperRuntimeImpl implements an interface for 
    
    - loading the shared library "af_bspwave_widget_win.dll" at runtime.
    - managing the BeSpice Wave Widget over text-based interactive commands
    - handling callbacks from BeSpice Wave. These callbacks call virtual 
        functions that can be handled by the integrating software.
        
    The class winBspWaveWrapperRuntime adds some convenience functions to this interface.
    These functions simplify the usage of commands sent to BeSpice Wave.

*/


class winBspWaveWrapperRuntimeImpl {

    // library handle
    void *m_libraryHandle;

    // function pointers
    win_bspwave_create_widget_type *m_createWidget;
    win_bspwave_process_command_line_type *m_processCommandLine;
    win_bspwave_execute_command_type *m_executeCommand;
    win_bspwave_get_command_result_type *m_getCommandResult;
    win_bspwave_register_callback_type *m_registerCallback;
    win_bspwave_create_and_run_main_type *m_createAndRunMain;
    win_bspwave_get_waveform_parser_type *m_getWaveformParser;
    win_bspwave_get_widget_info_type *m_getWidgetInfo;
    win_bspwave_register_win_callback_type *m_registerWinCallback;
    win_bspwave_register_style_callback_type *m_registerStyleCallback;

    HWND m_bspwaveWidget;
    bool m_flagDestroyObjectWhenClosing;

    std::string m_tmpString;
    
public:

    winBspWaveWrapperRuntimeImpl();
    virtual ~winBspWaveWrapperRuntimeImpl();

    // loads the dll at runtime and define the access functions for managing the BeSpice Wave Widget
    void loadBspWaveDll(const wchar_t *library_path);
    void loadBspWaveDll(const char *library_path);

    bool createWidget(HWND parent, int style_flags);
    bool createWidget(HWND parent, int x, int y, int w, int h, int style_flags);
    bool createMainWindow(int style_flags);

    HWND getBspwaveWidgetPtr() { return m_bspwaveWidget; }
    void setBspwaveWidgetPtr(HWND win) { m_bspwaveWidget = win; }

    std::string &tmpString() { return m_tmpString; }
    void setFlagDestroyObjectWhenClosing(bool f) {
        m_flagDestroyObjectWhenClosing = f;
    }

    void setGeometry(int x, int y, int w, int h);

    af_return_code cmdStd(const std::string &cmd);
    af_return_code cmdClose(bool flag_force);

    af_return_code processCommandLine(LPSTR cmd);

    const char **getCommandResult();

    virtual void onWidgetDestroyed(bool &flag_delete_object) {
        flag_delete_object = m_flagDestroyObjectWhenClosing;
        m_bspwaveWidget = 0;
    }

    // The following virtual functions are called by BeSpice Wave callbacks.
    virtual void onCloseButtonActivated() {}
    virtual void onAttachButtonActivated() {}
    virtual void onDetachButtonActivated() {}
    virtual int onNotify(const struct _af_bspwave_callback_notification_type *description);
    virtual void onCurveHighlighted(const std::string &section_name, const std::string &curve_name, int section_id, int curve_index, int flag_clear, int from_browser) {}
    virtual void onCurveUnHighlighted(const std::string &section_name, const std::string &curve_name, int section_id, int curve_index, int flag_clear, int from_browser) {}
    virtual void onCurveActivated(const std::string &section_name, const std::string &curve_name, int section_id, int curve_index, int flag_clear, int from_browser) {}
    virtual void onCheckIntegratorLicence(void *data) {}
    // (remark) The string "received_text" might be modified.
    //      Change received_text to an empty string tp prevent further actions.
    virtual bool onDropReceived(const std::string &plot_name, std::string &received_text) { return false; }
    virtual bool onGenerateDragString(const std::string &xml_string, std::string &drop_string) { return false; }
    virtual void onLoggerMessage(int severity, const std::string &message) {}
    virtual void onStatusBarMessage(int severity, const std::string &message) {}
    virtual void onProgressMessage(const std::string &message, int progress) {}
    virtual void onFileWasOpened(const std::string &message) {}
    virtual void onFileWasClosed(const std::string &message) {}
    virtual void onAllFilesWereClosed() {}
    virtual void onSectionUsedForLinkChanged() {}
    virtual int onOpenFileButtonActivated() { return 0; }

    // register or unregister callback functions. These functions will call tha above virtual functions.
    void registerCallbackFunctions(bool flag_drag_string_gene, bool flag_cursor_values);
    void unregisterCallbackFunctions();

    // register or unregister callback functions that allow to detach notebook pages. These functions will call the 
    //      virtual functions onTabCtrlPageAdded ...
    af_return_code registerWidgetCallbackFunctions();
    af_return_code unregisterWidgetCallbackFunctions();

    virtual void onTabCtrlPageAdded(const char *page_name, HWND page_handle) {}
    virtual void onTabCtrlPageRemoved(const char *page_name, HWND page_handle) {}
    virtual void onTabCtrlPageChanged(const char *page_name, HWND page_handle) {}
    virtual void onEnsurePageVisible(const char *page_name, HWND page_handle) {}

    // register or unregister a waveform parser. This parser must be implemented by the integrator.
    //      It might allow to parse special file types or to access data held in memory.
    af_return_code registerWaveformParserPlugin(struct _af_wp_plugin_functions_type *wp_plugin_functions);
    af_return_code unregisterWaveformParserPlugin(struct _af_wp_plugin_functions_type *wp_plugin_functions);

    // default handler for onNotify
    int handleNotification(const struct _af_bspwave_callback_notification_type *description);

private:

    void clearFunctionPointers();
    void loadFunctionPointers();
};

// this class adds some convenience functions to the above interface.
class winBspWaveWrapperRuntime : public bspWaveWrapper<winBspWaveWrapperRuntimeImpl> {

public:

    winBspWaveWrapperRuntime() : bspWaveWrapper<winBspWaveWrapperRuntimeImpl>() {
    }

};


#endif //__winWinBspWaveTest__
