/*
    Copyright Analog Flavor 2013-2024

    All rights reserved.

    This software contains proprietary and confidential information which is the
    property of Analog Flavor and is subject to license terms.

    This file may be used in accordance with the license agreement 
    provided with the software.

    This copyright notice may not be removed or altered.
*/


#include "stdafx.h"
#include <msclr/marshal_cppstd.h>
#include <string>

#include "cliWinBspWaveWrapper.h"

#include "winBspWaveWrapperRuntime.h"
#include "bspwave_plugin.h"


/*

    This class implements the virtual functions from winBspWaveWrapperRuntimeImpl.

    For each function an appropriate member of the interface class cliBspWaveWrapperInterface is called.

    The interface cliBspWaveWrapperInterface is implemented in cliWinBspWaveWrapper.

    */
public class winBspWaveWrapperRuntimeWithCallback : public winBspWaveWrapperRuntimeImpl {

    gcroot<cliBspWaveWrapperInterface ^> m_callback;

public:

    winBspWaveWrapperRuntimeWithCallback(cliBspWaveWrapperInterface ^cb) {
        m_callback = cb;
    }

    // re-implementation of virtual functions
    void onCloseButtonActivated() {
        m_callback->onCloseButtonActivated();
    }
    void onAttachButtonActivated() {
        m_callback->onAttachButtonActivated();
    }
    void onDetachButtonActivated() {
        m_callback->onDetachButtonActivated();
    }
    int onNotify(const struct _af_bspwave_callback_notification_type *description) {
        return m_callback->onNotify(description);
    }
    void onCurveHighlighted(const std::string &section_name, const std::string &curve_name, int section_id, int curve_index, int flag_clear, int from_browser) {
        String^ section = gcnew String(section_name.c_str());
        String^ curve = gcnew String(curve_name.c_str());
        m_callback->onCurveHighlighted(section, curve, section_id, curve_index, flag_clear, from_browser);
    }
    void onCurveUnHighlighted(const std::string &section_name, const std::string &curve_name, int section_id, int curve_index, int flag_clear, int from_browser) {
        String^ section = gcnew String(section_name.c_str());
        String^ curve = gcnew String(curve_name.c_str());
        m_callback->onCurveUnHighlighted(section, curve, section_id, curve_index, flag_clear, from_browser);
    }
    void onCurveActivated(const std::string &section_name, const std::string &curve_name, int section_id, int curve_index, int flag_clear, int from_browser) {
        String^ section = gcnew String(section_name.c_str());
        String^ curve = gcnew String(curve_name.c_str());
        m_callback->onCurveActivated(section, curve, section_id, curve_index, flag_clear, from_browser);
    }
    void onCheckIntegratorLicence(void *data) {
        m_callback->onCheckIntegratorLicence(data);
    }
    // (remark) The string "received_text" might be modified.
    //      Change received_text to an empty string to prevent further actions.
    bool onDropReceived(const std::string &plot_name, std::string &received_text) {
        String^ plot = gcnew String(plot_name.c_str());
        String^ text = gcnew String(received_text.c_str());
        if ( m_callback->onDropReceived(plot, text) ) {
            received_text = msclr::interop::marshal_as<std::string>(text);
            return true;
        }
        return false;
    }
    bool onGenerateDragString(const std::string &xml_string, std::string &drop_string) {
        String^ xml = gcnew String(xml_string.c_str());
        String^ drop = gcnew String(drop_string.c_str());
        if ( m_callback->onGenerateDragString(xml, drop) ) {
            drop_string = msclr::interop::marshal_as<std::string>(drop);
            return true;
        }
        return false;
    }
    void onLoggerMessage(int severity, const std::string &message) {
        String^ msg = gcnew String(message.c_str());
        m_callback->onLoggerMessage(severity, msg);
    }
    void onStatusBarMessage(int severity, const std::string &message) {
        String^ msg = gcnew String(message.c_str());
        m_callback->onStatusBarMessage(severity, msg);
    }
    void onProgressMessage(const std::string &message, int progress) {
        String^ msg = gcnew String(message.c_str());
        m_callback->onProgressMessage(msg, progress);
    }
    void onFileWasOpened(const std::string &message) {
        String^ msg = gcnew String(message.c_str());
        m_callback->onFileWasOpened(msg);
    }
    void onFileWasClosed(const std::string &message) {
        String^ msg = gcnew String(message.c_str());
        m_callback->onFileWasClosed(msg);
    }
    void onAllFilesWereClosed() {
        m_callback->onAllFilesWereClosed();
    }
    void onSectionUsedForLinkChanged() {
        m_callback->onSectionUsedForLinkChanged();
    }

};










cliWinBspWaveWrapper::cliWinBspWaveWrapper(const String ^library_path, int style_flags) {
    m_impl = new winBspWaveWrapperRuntimeWithCallback(this);
    m_styleFlags = style_flags;
    loadBspWaveDll(library_path);
}
cliWinBspWaveWrapper::cliWinBspWaveWrapper() {
    m_impl = new winBspWaveWrapperRuntimeWithCallback(this);
    m_styleFlags = 0;
}
cliWinBspWaveWrapper::~cliWinBspWaveWrapper() {
    if ( m_impl ) {
        delete m_impl;
        m_impl = 0;
    }
}
cliWinBspWaveWrapper::!cliWinBspWaveWrapper() {
    // Deallocate the native object on the finalizer just in case no destructor is called
    if ( m_impl ) {
        delete m_impl;
        m_impl = 0;
    }
}

HandleRef cliWinBspWaveWrapper::BuildWindowCore(HandleRef hwndParent) {
    HWND parent = (HWND)hwndParent.Handle.ToPointer();
    if ( m_impl && ::IsWindow(parent) ) {
        m_impl->createWidget(parent, 0, 0, 100, 100, m_styleFlags);
        if ( getBspwaveWidgetPtr() ) {
            onCreated();
        }
    }
    return HandleRef(this, IntPtr(getBspwaveWidgetPtr()));
}

void cliWinBspWaveWrapper::DestroyWindowCore(HandleRef hwnd) {
    // hwnd will be disposed for us
}


HWND cliWinBspWaveWrapper::getBspwaveWidgetPtr() {
    if ( m_impl ) {
        return m_impl->getBspwaveWidgetPtr();
    }
    return 0;
}
void cliWinBspWaveWrapper::loadBspWaveDll(const String ^library_path) {
    if ( m_impl ) {
        pin_ptr<const wchar_t> wch = PtrToStringChars(library_path);
        m_impl->loadBspWaveDll(wch);
    }
}
int cliWinBspWaveWrapper::cmdStd(String ^cmd) {
    if ( m_impl ) {
        std::string str = msclr::interop::marshal_as<std::string>(cmd);
        return m_impl->cmdStd(str);
    }
    return af_code_error;
}
int cliWinBspWaveWrapper::cmdClose(bool flag_force) {
    if ( m_impl ) {
        return m_impl->cmdClose(flag_force);
    }
    return af_code_error;
}
void cliWinBspWaveWrapper::registerCallbackFunctions(bool flag_drag_string_gene, bool flag_cursor_values) {
    if ( m_impl ) {
        m_impl->registerCallbackFunctions(flag_drag_string_gene, flag_cursor_values);
    }
}
void cliWinBspWaveWrapper::unregisterCallbackFunctions() {
}
/*
af_return_code cliWinBspWaveWrapper::registerWidgetCallbackFunctions() {
return af_code_error;
}
af_return_code cliWinBspWaveWrapper::unregisterWidgetCallbackFunctions() {
return af_code_error;
}
af_return_code cliWinBspWaveWrapper::registerWaveformParserPlugin(struct _af_wp_plugin_functions_type *wp_plugin_functions) {
return rc;
}
af_return_code cliWinBspWaveWrapper::unregisterWaveformParserPlugin(struct _af_wp_plugin_functions_type *wp_plugin_functions) {
af_return_code rc = af_code_error;
return rc;
}
*/
int cliWinBspWaveWrapper::onNotify(const struct _af_bspwave_callback_notification_type *description) {
    if ( m_impl ) {
        return m_impl->handleNotification(description);
    }
    return 0;
}



