/*
    Copyright Analog Flavor 2013-2024

    All rights reserved.

    This software contains proprietary and confidential information which is the
    property of Analog Flavor and is subject to license terms.

    This file may be used in accordance with the license agreement 
    provided with the software.

    This copyright notice may not be removed or altered.
*/

#include "qtWinBspWaveWrapper.h"

#include <windows.h>
#include "bspwave_win_widget.h"
#include "bspWaveWrapper.impl.h"
#include <sstream>

template class bspWaveWrapper<qtWinBspWaveWrapperImpl>;


qtWinBspWaveComponentWrapper::qtWinBspWaveComponentWrapper(QWidget *parent, HWND component, HWND main_win)
: QWidget(parent) {

    m_parentId = 0;
    m_oldComponentParent = 0;
    m_bspWaveComponent = component;
    m_bspWaveMainWindow = main_win;
    m_flagBlockEvents = false;

    handleNewSize();
    showComponent();
}

qtWinBspWaveComponentWrapper::~qtWinBspWaveComponentWrapper() {
    if ( m_oldComponentParent && m_bspWaveComponent ) {
        if ( ::IsWindow(m_oldComponentParent) && ::IsWindow(m_bspWaveComponent) ) {
            ::SetParent(m_bspWaveComponent, m_oldComponentParent);

            if ( m_bspWaveMainWindow && ::IsWindow(m_bspWaveMainWindow) ) {
                win_bspwave_execute_command(m_bspWaveMainWindow, "bspwave_components_were_reparented");
            }
        }
    }
}

void qtWinBspWaveComponentWrapper::setBspWaveComponent(HWND w) {
    if ( m_bspWaveComponent == 0 ) {
        m_bspWaveComponent = w;
        handleNewSize();
    }
}

void qtWinBspWaveComponentWrapper::showEvent(QShowEvent * event) {
    create();
    handleNewSize();
}

void qtWinBspWaveComponentWrapper::resizeEvent(QResizeEvent * event) {
    if ( m_bspWaveComponent ) {
        int w, h;
        w = event->size().width();
        h = event->size().height();
        handleNewSize(w, h);
    }
}

bool qtWinBspWaveComponentWrapper::event(QEvent *event) {
    if ( event->type() == QEvent::WinIdChange ) {
        if ( !m_flagBlockEvents ) {
            reparent();
        }
    }
    return QWidget::event(event);
}

void qtWinBspWaveComponentWrapper::reparent() {
    if ( m_bspWaveComponent ) {
        m_flagBlockEvents = true;
        /// this simply makes sure the wrapper is a native widget
        QWidget::winId();
        WId parent_id = QWidget::effectiveWinId();
        if ( parent_id != m_parentId ) {
            if ( m_oldComponentParent == 0 ) {
                m_oldComponentParent = ::GetParent(m_bspWaveComponent);
            }
            m_parentId = parent_id;
            HWND parent_hwnd = reinterpret_cast<HWND>( m_parentId );
            ::SetParent(m_bspWaveComponent, parent_hwnd);

            if ( m_bspWaveMainWindow && ::IsWindow(m_bspWaveMainWindow) ) {
                win_bspwave_execute_command(m_bspWaveMainWindow, "bspwave_components_were_reparented");
            }
        }
        m_flagBlockEvents = false;
    }
}

void qtWinBspWaveComponentWrapper::handleNewSize(int w, int h) {
    if ( m_bspWaveComponent ) {
        int x, y;
        x = 0;
        y = 0;
        reparent();
        if ( m_parentId ) {
            QWidget *qp = QWidget::find(m_parentId);
            if ( qp ) {
                /// screen pos of current widget
                QPoint p = mapToGlobal(QPoint(0, 0));
                x = p.x();
                y = p.y();
                /// screen pos of parent widget
                p = qp->mapToGlobal(QPoint(0, 0));
                x = x - p.x();
                y = y - p.y();
            }
        }
        ::MoveWindow(m_bspWaveComponent, x, y, w, h, 1);
    }
}

void qtWinBspWaveComponentWrapper::handleNewSize() {
    int w, h;
    w = width();
    h = height();
    handleNewSize(w, h);
}

void qtWinBspWaveComponentWrapper::showComponent() {
    if ( m_bspWaveComponent ) {
        ::ShowWindow(m_bspWaveComponent, SW_SHOW);
    }
}

void qtWinBspWaveComponentWrapper::hideComponent() {
    if ( m_bspWaveComponent ) {
        ::ShowWindow(m_bspWaveComponent, SW_HIDE);
    }
}





qtWinBspWaveWrapperImpl::qtWinBspWaveWrapperImpl() 
: qtWinBspWaveComponentWrapper(0, 0, 0) {

    m_style = 0;
}

qtWinBspWaveWrapperImpl::qtWinBspWaveWrapperImpl(QWidget *parent, int style)
: qtWinBspWaveComponentWrapper(parent, 0, 0) {

    m_style = style;
}
void qtWinBspWaveWrapperImpl::create() {
    if ( getBspwaveWidgetPtr() == 0 ) {
        /// this simply makes sure the wrapper is a native widget
        QWidget::winId();
        WId parent_id = QWidget::effectiveWinId();
        if ( parent_id ) {
            HWND parent_hwnd = reinterpret_cast<HWND>( parent_id );
            createWidget(parent_hwnd, 0, 0, 1000, 1000, m_style);
            setBspWaveComponent(getBspwaveWidgetPtr());
            setParentId(parent_id);
            showBspwave();
        }
    }
    else {
        reparent();
    }
}
af_return_code qtWinBspWaveWrapperImpl::cmdQt(const QString &cmd) {
    std::string std_cmd = cmd.toStdString();
    return cmdStd(std_cmd);
}
af_return_code qtWinBspWaveWrapperImpl::cmdOpenFilesQt(const QStringList &files) {
    af_return_code rc = af_code_ok;
    af_return_code rc_cmd_exec;
    QString str_cmd;
    QStringList::ConstIterator it = files.begin();
    while( it != files.end() ) {
        str_cmd = QString("open_file \"") + *it + QString("\"\n");
        rc_cmd_exec = cmdQt(str_cmd);
        if ( rc_cmd_exec != af_code_ok ) {
            rc = rc_cmd_exec;
        }
        ++it;
    }
    return rc;
}



