/*
    Copyright Analog Flavor 2013-2024

    All rights reserved.

    This software contains proprietary and confidential information which is the
    property of Analog Flavor and is subject to license terms.

    This file may be used in accordance with the license agreement 
    provided with the software.

    This copyright notice may not be removed or altered.
*/

/*
#
#  Copyright Analog Flavor and Gerard Eichenmueller 2013
#
#  All rights reserved.
#
#  This software contains proprietary and confidential information which is the
#  property of Analog Flavor and Gerard Eichenmueller and is subject to license terms.
#
*/

#include "tkWinBspWaveWrapper.h"

#include <Windows.h>
#include "bspwave_win_widget.h"

#include <tkPlatDecls.h>

#include <sstream>
#include <algorithm>
#include <map>
using namespace std;

/** 
    Headers for functions required for usage in a tcl tk script.
*/
extern "C" {
    AF_TK_WIN_BSP_API int Af_bspwave_widget_tk_win_Init(Tcl_Interp *interp);
    AF_TK_WIN_BSP_API int tk_win_create_bspwave_widget_object(ClientData client_data, Tcl_Interp *interp, int objc, Tcl_Obj *const objv[]);
    AF_TK_WIN_BSP_API int tk_win_bspwave_widget_command(ClientData client_data, Tcl_Interp *interp, int objc, Tcl_Obj *const objv[]);
    AF_TK_WIN_BSP_API void tk_win_delete_bspwave_widget_object(ClientData client_data);
}

/// This structure is used to register the created wrappers and associate them to a tk window.
static std::map<Tk_Window, tkWinBspWaveWrapper*> static_registered_widgets;


/// tk event handler
void tk_win_af_widget_event_handler_structure( ClientData client_data, XEvent *event_ptr);

void tk_win_af_widget_event_handler_structure( ClientData client_data, XEvent *event_ptr) {
    tkWinBspWaveWrapper *win = (tkWinBspWaveWrapper*)(client_data);
    win->processStructureEvent(event_ptr);
}


tkWinBspWaveWrapper::tkWinBspWaveWrapper(Tcl_Interp *interp, Tk_Window main_win, const char *window_path, int x, int y, int width, int height, int style_flags) {

    m_bspwave = 0;
    m_styleFlags = style_flags;
    m_tkWin = Tk_CreateWindowFromPath(interp, main_win, window_path, 0);

    int event_mask = StructureNotifyMask;
    Tk_CreateEventHandler( m_tkWin, event_mask, tk_win_af_widget_event_handler_structure, this);

    Tk_GeometryRequest(m_tkWin, width, height);
    static_registered_widgets.insert(make_pair(m_tkWin, this));

    /// registers the window to make sure it receives commands
    Tcl_CreateObjCommand(interp, window_path, tk_win_bspwave_widget_command, m_tkWin, tk_win_delete_bspwave_widget_object);

    m_interpreter = interp;
    m_windowName = std::string(window_path);
}
tkWinBspWaveWrapper::~tkWinBspWaveWrapper() {
    static_registered_widgets.erase(m_tkWin);
    Tcl_DeleteCommand(m_interpreter, m_windowName.c_str());
}
void tkWinBspWaveWrapper::createBspwave() {
    Window x_win;
    HWND hwnd_parent;
    int w, h;
    if ( m_bspwave == 0 ) {
        x_win = Tk_WindowId(m_tkWin);
        hwnd_parent = Tk_GetHWND(x_win);
        if ( hwnd_parent ) {
            w = Tk_Width(m_tkWin);
            h = Tk_Height(m_tkWin);
            m_bspwave = win_bspwave_create_widget(hwnd_parent, 0, 0, w, h, m_styleFlags);
            if ( m_bspwave ) {
                if ( !m_commandLineArgument.empty() ) {
                    win_bspwave_process_command_line(m_bspwave, (LPSTR)( m_commandLineArgument.c_str() ));
                    m_commandLineArgument.clear();
                }
                for ( std::list<std::string>::const_iterator it = m_pendingCommands.begin(); it != m_pendingCommands.end(); it++ ) {
                    const std::string &cmd = *it;
                    win_bspwave_execute_command(m_bspwave, cmd.c_str());
                }
                m_pendingCommands.clear();
            }
        }
    }
}
int tkWinBspWaveWrapper::processStructureEvent(XEvent *event_ptr) {
    int w, h;
    XConfigureEvent *conf_event_ptr;

    switch ( event_ptr->type ) {
    case ConfigureNotify:
        conf_event_ptr = &(event_ptr->xconfigure);
        w = 1;
        h = 1;
        if ( conf_event_ptr->width > 0 ) {
            w = conf_event_ptr->width;
        }
        if ( conf_event_ptr->height > 0 ) {
            h = conf_event_ptr->height;
        }
        if ( m_bspwave ) {
            win_bspwave_move_widget(m_bspwave, 0, 0, w, h);
        }
        return 1;
    case MapNotify:
        createBspwave();
        return 1;
    default:
        break;
    }
    return 0;
}


/** 
    Implementation of functions required for usage in a tcl tk script.
*/

/// creating a wrapper window
int tk_win_create_bspwave_widget_object(ClientData client_data, Tcl_Interp *interp, int objc, Tcl_Obj *const objv[]) {

    Tk_Window tkwin;
    tkWinBspWaveWrapper *bspwave_widget = 0;

    if (objc < 2) {
        Tcl_WrongNumArgs(interp, 1, objv, "pathName ?-option value ...?");
        return TCL_ERROR;
    }

    Tk_Window main_win = Tk_MainWindow(interp);
    char *window_path = Tcl_GetString(objv[1]);

    long int style_flags = 0;
    int width = 100;
    int height = 100;
    int arg_i = 2;
    while ( arg_i < objc ) {
        const char *arg_val = Tcl_GetString(objv[arg_i]);
        if ( arg_val ) {
            std::string cmd(arg_val);
            std::transform(cmd.begin(), cmd.end(), cmd.begin(), ::tolower);
            if ( cmd == "-style" ) {
                arg_i++;
                arg_val = Tcl_GetString(objv[arg_i]);
                if ( arg_val ) {
                    style_flags = strtol(arg_val, 0, 10);
                }
            }
            else if ( cmd == "-width" ) {
                arg_i++;
                arg_val = Tcl_GetString(objv[arg_i]);
                if ( arg_val ) {
                    width = strtol(arg_val, 0, 10);
                }
            }
            else if ( cmd == "-height" ) {
                arg_i++;
                arg_val = Tcl_GetString(objv[arg_i]);
                if ( arg_val ) {
                    height = strtol(arg_val, 0, 10);
                }
            }
        }
        arg_i++;
    }

    if ( width <= 0 ) {
        width = 1;
    }
    if ( height <= 0 ) {
        height = 1;
    }

    try {
        bspwave_widget = new tkWinBspWaveWrapper(interp, main_win, window_path, 0, 0, width, height, style_flags);
    }
    catch ( std::exception &ex ) {
        bspwave_widget = 0;
    }

    if ( bspwave_widget == 0 ) {
        return TCL_ERROR;
    }
    tkwin = bspwave_widget->getTkWindow();

    return TCL_OK;
}

/// deleting the wrapper window
void tk_win_delete_bspwave_widget_object(ClientData client_data) {
    Tk_Window tk_win = (Tk_Window)(client_data);
    tkWinBspWaveWrapper *bspwave_widget = 0;

    std::map<Tk_Window, tkWinBspWaveWrapper*>::iterator it = static_registered_widgets.find(tk_win);
    if ( it != static_registered_widgets.end() ) {
        bspwave_widget = it->second;
    }
    if ( bspwave_widget != 0 ) {
        delete bspwave_widget;
    }
}

/// sending an interactive command to Bespice Wave
int tk_win_bspwave_widget_command(ClientData client_data, Tcl_Interp *interp, int objc, Tcl_Obj *const objv[]) {
    if (objc < 1 || ! client_data ) {
        Tcl_WrongNumArgs(interp, 1, objv, "pathName ?-option value ...?");
        return TCL_ERROR;
    }
    Tk_Window tk_win = (Tk_Window)(client_data);
    tkWinBspWaveWrapper *bspwave_widget = 0;

    std::map<Tk_Window, tkWinBspWaveWrapper*>::iterator it = static_registered_widgets.find(tk_win);
    if ( it != static_registered_widgets.end() ) {
        bspwave_widget = it->second;
    }
    if ( bspwave_widget == 0 ) {
        return TCL_ERROR;
    }

    size_t arg_i = 1;
    std::string inter_cmd;
    while ( arg_i < objc ) {
        const char *arg_val = Tcl_GetString(objv[arg_i]);
        if ( arg_val ) {
            std::string cmd(arg_val);
            std::transform(cmd.begin(), cmd.end(), cmd.begin(), ::tolower);
            if ( cmd == "execute_command" ) {
                arg_i++;
                arg_val = Tcl_GetString(objv[arg_i]);
                if ( arg_val ) {
                    inter_cmd = std::string(arg_val);
                }
            }
        }
        arg_i++;
    }
    af_return_code rc;
    if ( ! inter_cmd.empty() ) {
        HWND win = bspwave_widget->getHWND();
        if ( win ) {
            rc = win_bspwave_execute_command(win, inter_cmd.c_str());
        }
        else {
            bspwave_widget->pendingCommands().push_back(inter_cmd);
            rc = af_code_queued;
        }
        if ( rc == af_code_ok ) {
            const char **ret_val = win_bspwave_get_command_result(bspwave_widget->getHWND());
            if ( ret_val == 0 ) {
                Tcl_SetResult(interp, "command succeeded", 0);
            }
            else if ( ret_val[0] == 0 ) {
                Tcl_SetResult(interp, "command succeeded", 0);
            }
            else if ( ret_val[1] == 0 ) {
                Tcl_SetResult(interp, (char*)(ret_val[0]), 0);
            }
            else {
                Tcl_Obj *list_obj = Tcl_NewListObj(0, NULL);
                while ( *ret_val ) {
                    Tcl_ListObjAppendElement(interp, list_obj, Tcl_NewStringObj(*ret_val, -1));
                    ret_val++;
                }
                Tcl_SetObjResult(interp, list_obj);
            }
            return TCL_OK;
        }
        if ( rc == af_code_queued ) {
            Tcl_SetResult(interp, "command queued", 0);
            return TCL_OK;
        }
        Tcl_SetResult(interp, "command failed", 0);
        return TCL_OK;
    }
    return TCL_ERROR;
}

/// this function is called by TK to initialize the library
int Af_bspwave_widget_tk_win_Init(Tcl_Interp *interp) {

#ifdef USE_TCL_STUBS
    if ( Tcl_InitStubs(interp, TCL_VERSION, 0) == NULL) {
        return TCL_ERROR;
    }
    if ( Tk_InitStubs(interp, TCL_VERSION, 0) == NULL) {
        return TCL_ERROR;
    }
#else
    if ( Tcl_PkgRequire(interp, "Tcl", TCL_VERSION, 0) == NULL) {
        return TCL_ERROR;
    }
    if ( Tcl_PkgRequire(interp, "Tk", TCL_VERSION, 0) == NULL) {
        return TCL_ERROR;
    }
#endif

    /// creating a tcl command for creating a wrapper window
    Tcl_CreateObjCommand(interp, "af_win_bspwave_widget", tk_win_create_bspwave_widget_object, NULL, NULL);
    return TCL_OK;
}

/**
        Implementation of C interface functions defined in bspwave_tk_win_widget.h
*/
int win_tk_bspwave_initialize_library(Tcl_Interp *interp) {
    return Af_bspwave_widget_tk_win_Init(interp);
}
Tk_Window win_tk_bspwave_create_widget_from_path(Tcl_Interp *interp,
                Tk_Window main_win, const char *window_path,
                int x, int y, int width, int height, int style_flags) {
    try { 
        tkWinBspWaveWrapper *wrapper = new tkWinBspWaveWrapper(interp, main_win, window_path, x, y, width, height, style_flags);
        return wrapper->getTkWindow();
    }
    catch ( std::exception &ex ) {
    }
    return 0;
}
af_return_code win_tk_bspwave_process_command_line(Tk_Window bspwave_widget, int argc, char **argv) {
    std::stringstream cmd_line;
    for ( int i = 1; i < argc; i++ ) {
        if ( argv[i] ) {
            cmd_line << " " << argv[i];
        }
    }
    std::map<Tk_Window, tkWinBspWaveWrapper*>::iterator it = static_registered_widgets.find(bspwave_widget);
    if ( it != static_registered_widgets.end() ) {
        tkWinBspWaveWrapper *wrapper = it->second;
        if ( wrapper ) {
            HWND bsp_hwnd = wrapper->getHWND();
            if (bsp_hwnd) {
                return win_bspwave_process_command_line(bsp_hwnd, (LPSTR)(cmd_line.str().c_str()));
            }
            else {
                /// bspwave has not yet been created
                wrapper->setCommandLineArgument(cmd_line.str());
                return af_code_ok;
            }
        }
    }
    return af_code_error;
}
af_return_code win_tk_bspwave_execute_command(Tk_Window bspwave_widget, const char *command) {
    std::map<Tk_Window, tkWinBspWaveWrapper*>::iterator it = static_registered_widgets.find(bspwave_widget);
    if ( it != static_registered_widgets.end() ) {
        tkWinBspWaveWrapper *wrapper = it->second;
        if ( wrapper ) {
            HWND bsp_hwnd = wrapper->getHWND();
            return win_bspwave_execute_command(bsp_hwnd, command);
        }
    }
    return af_code_error;
}
const char **win_tk_bspwave_get_command_result(Tk_Window bspwave_widget) {
    std::map<Tk_Window, tkWinBspWaveWrapper*>::iterator it = static_registered_widgets.find(bspwave_widget);
    if ( it != static_registered_widgets.end() ) {
        tkWinBspWaveWrapper *wrapper = it->second;
        if ( wrapper ) {
            HWND bsp_hwnd = wrapper->getHWND();
            return win_bspwave_get_command_result(bsp_hwnd);
        }
    }
    return 0;
}
af_return_code win_tk_bspwave_register_callback(Tk_Window bspwave_widget, const af_bspwave_callback_type *callbacks) {
    std::map<Tk_Window, tkWinBspWaveWrapper*>::iterator it = static_registered_widgets.find(bspwave_widget);
    if ( it != static_registered_widgets.end() ) {
        tkWinBspWaveWrapper *wrapper = it->second;
        if ( wrapper ) {
            HWND bsp_hwnd = wrapper->getHWND();
            return win_bspwave_register_callback(bsp_hwnd, callbacks);
        }
    }
    return af_code_error;
}

